/*

Copyright 2019, dettus@dettus.net

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


*/

#include <stdio.h>
#include <stdlib.h>
#include "default_render.h"
#include "picture.h"
int default_render_lowansi(tPicture* picture,int rows,int cols)
{
	int i;
	int j;
	int k,l;

	int y_up,y_down,x_left,x_right;
	int accux,accuy;
	int cnt2;
	int red,green,blue;
	int p;
	int col;
	char c;
	accux=accuy=0;
	
	y_up=0;
	c='$';
	for (i=0;i<picture->height;i++)
	{
		accuy+=rows;
		if (accuy>=picture->height || i==picture->height-1)
		{
			accuy-=picture->height;
			y_down=i+1;
			x_left=0;
			for (j=0;j<picture->width;j++)
			{
				accux+=cols;
				if (accux>=picture->width || j==picture->width-1)
				{
					x_right=j+1;
					accux-=picture->width;		
					red=green=blue=0;
					cnt2=0;
					for (k=y_up;k<y_down;k++)
					{
						for (l=x_left;l<x_right;l++)
						{
							p=picture->pixels[k*picture->width+l];
							red+=(picture->palette[p]>>8)&0x7;	// red
							green+=(picture->palette[p]>>4)&0x7;	// green
							blue+=(picture->palette[p]>>0)&0x7;	// blue
							cnt2++;
						}
					}

					if ((red==blue) && (red==green))        // all colors have the same intensity. so: grey scale                    
					{       
						if (red==0) col=0;   // black                                                
						else if (red>0 && red<2) col=8;   // dark grey                                    
						else if (red>=2 && red<4) col=7;  // light grey;
						else col=15;  // bright white                                         

					} else if (red>green && red>blue) {     // red is the most intense color
						if (red<4) col=1;    // dark red                                             
						else col=9;          // bright red                                           

					} else if (green>red && green>blue) {   // green is the most intense color              
						if (green<3) col=2;  // dark green
						else col=10;         // bright green
					} else if (blue>red && blue>green) {    // blue is the most intense color               
						if (blue<3) col=4;   // dark blue
						else col=12;         // bright blue
					} else if (red==green)  // brown or yellow                                              
					{               
						if (red<4) col=3;  
						else col=11;                                                                 
					} else if (red==blue)   // magenta                                                      
					{
						if (red<3) col=5;
						else col=13;                                                                 
					} else if (green==blue) // cyan
					{                                                                                       
						if (green<3) col=6;                                                          
						else col=14;
					}
					printf("\x1b[%d;3%dm%c",col/8,col%8,c);

					x_left=x_right;
				}
			}
			y_up=y_down;
			printf("\x1b[0m\n");
		}
	}

}
int default_render_monochrome(tPicture* picture,int rows,int cols)
{
	int i;
	int j;
	int k,l;

	int y_up,y_down,x_left,x_right;
	int accux,accuy;
	int grey;
	int mingrey,maxgrey;
	int cnt;
	int cnt2;
	int p;
	const unsigned char greyscales[14]=" .:-=+*x#/@$X";	// those characters increase in density.
	const int scalenum=13;
	accux=accuy=0;
	mingrey=maxgrey=0;
	// first: try to find the brightest/darkest pixels
	
	y_up=0;
	cnt=0;
	for (i=0;i<picture->height;i++)
	{
		accuy+=rows;
		if (accuy>=picture->height || i==picture->height-1)
		{
			accuy-=picture->height;
			y_down=i+1;
			x_left=0;
			for (j=0;j<picture->width;j++)
			{
				accux+=cols;
				if (accux>=picture->width || j==picture->width-1)
				{
					x_right=j+1;
					accux-=picture->width;		
					// at this point, a rectangle between y_up,y_down, x_left,x_right contains the pixels that need to be greyscaled.
					grey=0;
					cnt2=0;
					for (k=y_up;k<y_down;k++)
					{
						for (l=x_left;l<x_right;l++)
						{
							p=picture->pixels[k*picture->width+l];
							grey+=(picture->palette[p]>>8)&0x7;	// red
							grey+=(picture->palette[p]>>4)&0x7;	// green
							grey+=(picture->palette[p]>>0)&0x7;	// blue
							cnt2++;
						}
					}
					grey/=cnt2;
						
					if (cnt==0 || grey>maxgrey) maxgrey=grey;
					if (cnt==0 || grey<mingrey) mingrey=grey;
					cnt++;
					x_left=x_right;
				}
			}
			y_up=y_down;
		}
	}
	// mingrey is the lowest grey scale value.
	// maxgrey is the highest one.


	y_up=0;
	accuy=0;
	for (i=0;i<picture->height;i++)
	{
		accuy+=rows;
		if (accuy>=picture->height || i==picture->height-1)
		{
			accuy-=picture->height;
			y_down=i+1;
			x_left=0;
			for (j=0;j<picture->width;j++)
			{
				accux+=cols;
				if (accux>=picture->width || j==picture->width-1)
				{
					x_right=j+1;
					accux-=picture->width;
					
					// at this point, a rectangle between y_up,y_down, x_left,x_right contains the pixels that need to be greyscaled.
					grey=0;
					cnt2=0;
					for (k=y_up;k<y_down;k++)
					{
						for (l=x_left;l<x_right;l++)
						{
							p=picture->pixels[k*picture->width+l];
							grey+=(picture->palette[p]>>8)&0x7;	// red
							grey+=(picture->palette[p]>>4)&0x7;	// green
							grey+=(picture->palette[p]>>0)&0x7;	// blue
							cnt2++;
						}
					}
					grey/=cnt2;
					grey-=mingrey;
					grey*=(scalenum-1);
					grey/=(maxgrey-mingrey);
					printf("%c",greyscales[grey]);
					x_left=x_right;
				}
			}
			y_up=y_down;
			printf("\n");
		}
	}
	return 0;
}

